<?php

namespace Drupal\simple_integrations;

use GuzzleHttp\Client;
use Drupal\simple_integrations\Exception\IntegrationInactiveException;

/**
 * The connection client.
 *
 * Acts as an extension of core's http_client service, with additional
 * functionality to configure the request automatically.
 */
class ConnectionClient extends Client {

  /**
   * The associated Integration.
   *
   * @var \Drupal\simple_integrations\IntegrationInterface
   */
  protected $integration;

  /**
   * The authentication type.
   *
   * @var string
   */
  protected $authType;

  /**
   * The authentication username.
   *
   * @var string
   */
  protected $authUser;

  /**
   * The authentication password or key.
   *
   * @var string
   */
  protected $authKey;

  /**
   * The request end point.
   *
   * @var string
   */
  public $endPoint;

  /**
   * Config for the connection.
   *
   * @var array
   */
  public $config = [];

  /**
   * Send a GET request to an endpoint.
   *
   * This wraps the GuzzleHttp Client's get() magic method, which in turn
   * performs a request() call.
   *
   * Prior to performing the GET request, check to see if the linked integration
   * is active. If it isn't, exit out with an exception.
   *
   * @param string $uri
   *   The endpoint to request.
   * @param array $options
   *   An array of options.
   *
   * @throws \Drupal\simple_integrations\Exception\IntegrationInactiveException
   */
  public function get($uri, array $options) {
    if (!$this->integration->isActive()) {
      throw new IntegrationInactiveException($this->integration->id());
    }

    return parent::get($uri, $options);
  }

  /**
   * Set the integration entity.
   *
   * @param \Drupal\simple_integrations\IntegrationInterface $integration
   *   The integration entity.
   *
   * @return \Drupal\simple_integrations\ConnectionClient
   *   Return this object.
   */
  public function setIntegration(IntegrationInterface $integration) {
    $this->integration = $integration;
    return $this;
  }

  /**
   * Get the integration entity.
   *
   * @return \Drupal\simple_integrations\IntegrationInterface
   *   The integration entity.
   */
  public function getIntegration() {
    return $this->integration;
  }

  /**
   * Perform configuration tasks.
   */
  public function configure() {
    // Configure the authentication. Always add these values, even if they're
    // blank.
    $this
      ->setAuthType()
      ->setAuthValues();

    // Configure the request end point.
    $this->setRequestEndPoint();

    // Add the credentials to the request, if appropriate.
    if ($this->authType != 'none') {
      $this->setCredentials();
    }
  }

  /**
   * Set authentication type from a given integration.
   *
   * @return \Drupal\simple_integrations\ConnectionClient
   *   Return this object.
   */
  public function setAuthType() {
    $this->authType = $this->integration->get('auth_type');

    return $this;
  }

  /**
   * Set the authentication values from a given integration.
   *
   * @return \Drupal\simple_integrations\ConnectionClient
   *   Return this object.
   */
  public function setAuthValues() {
    $this->authUser = $this->integration->get('auth_user');
    $this->authKey = $this->integration->get('auth_key');

    return $this;
  }

  /**
   * Set the end point for this request.
   *
   * @return \Drupal\simple_integrations\ConnectionClient
   *   Return this object.
   */
  public function setRequestEndPoint() {
    $this->endPoint = $this->integration->get('external_end_point');

    return $this;
  }

  /**
   * Add credentials to a request.
   *
   * @return \Drupal\simple_integrations\ConnectionClient
   *   Return this object.
   */
  public function setCredentials() {
    if ($this->authType == 'headers') {
      // Add authentication headers.
      $this->config['headers']['Authorization'] = [
        $this->authUser,
        $this->authKey,
      ];
    }
    elseif ($this->authType == 'basic_auth') {
      // Add basic authentication.
      $this->config['auth'] = [
        $this->authUser,
        $this->authKey,
      ];
    }

    return $this;
  }

}
