<?php

namespace Drupal\Tests\simple_integrations\Functional;

use Drupal\Tests\BrowserTestBase;
use Drupal\simple_integrations\ConnectionClient;

/**
 * Validation tests for Simple Integrations.
 *
 * @group simple_integrations
 */
class IntegrationsTest extends BrowserTestBase {

  /**
   * Modules to enable.
   *
   * @var array
   */
  public static $modules = ['simple_integrations'];

  /**
   * An integration.
   *
   * @var \Drupal\simple_integrations\IntegrationInterface
   */
  public $integration;

  /**
   * A connection client.
   *
   * @var \Drupal\simple_integrations\ConnectionClient
   */
  public $connection;

  /**
   * Setup.
   */
  public function setUp() {
    parent::setUp();

    // Create a new integration.
    $entity_storage = \Drupal::entityTypeManager()->getStorage('integration');
    $integration = $entity_storage->create([
      'id' => 'test_integration',
      'label' => 'Test Integration',
      'active' => TRUE,
      'debug_mode' => FALSE,
      'external_end_point' => 'https://google.co.uk',
      'auth_type' => ['none' => 'None'],
    ]);
    $integration->save();
    $this->integration = $integration;

    // Create a connection client.
    $connection = new ConnectionClient();
    $connection->setIntegration($this->integration);
    $connection->configure();
    $this->connection = $connection;
  }

  /**
   * Given an integration, test that a connection can be made.
   */
  public function testActiveIntegrationConnection() {
    $this->integration->performConnectionTest($this->connection);
    $this->assertSession()->statusCodeEquals(200);
  }

  /**
   * Given an inactive integration, test that no connection can be made.
   *
   * @expectedException IntegrationInactiveException
   */
  public function testInactiveIntegrationConnection() {
    $this->integration->set('active', FALSE);
    $this->integration->save();

    $this->integration->performConnectionTest($this->connection);
  }

  /**
   * Given an integration, test that a valid debug message can be created.
   *
   * @dataProvider debugMessageGoodDataProvider
   */
  public function testGoodDebugMessages($message, $type) {
    $this->integration->set('active', TRUE);
    $this->integration->set('debug_mode', TRUE);

    $this->integration->logDebugMessage($message, $type);
  }

  /**
   * Given an empty debug message, ensure no debug message is created.
   *
   * @dataProvider debugMessageBadDataProvider()
   *
   * @expectedException EmptyDebugMessageException
   *
   * @see debugMessageBadDataProvider()
   */
  public function testEmptyDebugMessages($message, $type) {
    $this->integration->logDebugMessage($message, $type);
  }

  /**
   * Given an integration, ensure no message is created when not in debug mode.
   *
   * @dataProvider debugMessageGoodDataProvider()
   *
   * @expectedException DebugModeDisabledException
   *
   * @see debugMessageGoodDataProvider()
   */
  public function testDebugModeDisabled($message, $type) {
    $this->integration->set('debug_mode', FALSE);
    $this->integration->logDebugMessage($message, $type);
  }

  /**
   * Valid debug message data provider.
   *
   * @see testGoodDebugMessages()
   * @see testDebugModeDisabled()
   */
  public function debugMessageGoodDataProvider() {
    return [
      ['Test debug message', 'notice'],
      ['There was an error', 'error'],
    ];
  }

  /**
   * Invalid debug message data provider.
   *
   * @see testEmptyDebugMessages()
   */
  public function debugMessageBadDataProvider() {
    return [
      ['', 'error'],
      [[], 'error'],
    ];
  }

}
